// mozileConfiguration.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *		Lars Kiilerich   <lars[at]v1p[dot]dk>
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Configuration Javascript
Mozile version 0.6.14

Created by James A. Overton
Created on 2004-03-24

Last revised by James A. Overton
Last revised on 2004-06-24

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, while the 
majority was cut out of mozileTB.js and separated into different files.


DESCRIPTION:
	Injects the eDOM code into the current when told to.

*********/


// Mozile Inject
// Load eDOM and related files into the target document, just like mozileChromeLoader.js

//List of files to inject.
var mozile_js_files = new Array();
mozile_js_files = ["eDOM.js", "eDOMXHTML.js", "domlevel3.js", "mozCE.js", "mozIECE.js"];

function mozileInject(mozileInjectTarget) {
	var f = ["mozileConfiguration.js","mozileInject"];
	mozileDebug(f,2,"Injection starting");
	
	try {
  	var injectPosn;
		if ( mozileInjectTarget.document.documentElement.nodeName.toLowerCase() == "html" ) {
			if ( mozileInjectTarget.document.getElementsByTagName("head").length > 0 ) {
				injectPosn = mozileInjectTarget.document.getElementsByTagName("head")[0];
				mozileDebug(f,3,"Injecting into X/HTML");
			}
			//else if no head???
		} else {
			injectPosn = mozileInjectTarget.document.documentElement;
			mozileDebug(f,3,"Injecting into XML");
		}
		mozileLoadConfig(mozileInjectTarget, injectPosn);
		mozileInjectFiles( mozileInjectTarget );	
		mozileInjectTarget.addEventListener("beforeunload", mozileOnBeforeUnload, false);
		// Global flag to let you know Mozile has been loaded from chrome.
		mozileInjectTarget.mozileChromeLoaded = "true";
		mozileDebug(f,2,"Injection successful");
	} catch (e) {
		mozileDebug(f,1,"Injection error: "+e);
	}
}

// Add the script files etc required by Mozile.
function mozileInjectFiles( target ) {
	var f = ["mozileConfiguration.js","mozileInjectFiles"];
	var docEl, head, tag, src, i;
	try {
	docEl = target.document.documentElement;
		if ( docEl.nodeName == "HTML" ) { //...assume XHTML compatible or HTML, so namespace not required.
			//If no head, need to create one?
			head = target.document.getElementsByTagName("head")[0];
			for (i=0; i < mozile_js_files.length; i++) {
				tag = target.document.createElement("script");
				src = MOZILEC.EXTENSION_DIR + mozile_js_files[i];
				tag.setAttribute("id", mozile_js_files[i]);
				tag.setAttribute("src", src);
				tag.setAttribute("language","JavaScript");
				head.appendChild(tag);
			}
		}else if ( docEl.nodeName == "html" ) { //...assume XHTML
			//If no head, need to create one?
			head = target.document.getElementsByTagName("head")[0];
			for (i=0; i < mozile_js_files.length; i++) {
				tag = target.document.createElementNS(MOZILEC.NAMESPACE_XHTML,"script");
				src = MOZILEC.EXTENSION_DIR + mozile_js_files[i];
				tag.setAttribute("id", mozile_js_files[i]);
				tag.setAttribute("src", src);
				tag.setAttribute("language","JavaScript");
				head.appendChild(tag);
			}
		} else { //...assume XML
			for (i=0; i < mozile_js_files.length; i++) {
				tag = target.document.createElementNS(MOZILEC.NAMESPACE_XHTML,"script");
				src = MOZILEC.EXTENSION_DIR + mozile_js_files[i];
				tag.setAttribute("id", mozile_js_files[i]);
				tag.setAttribute("src", src);
				tag.setAttribute("language","JavaScript");
				docEl.insertBefore( tag, docEl.firstChild );
			}
		}
	} catch (e) {
		mozileDebug(f,1,"Error injecting files: "+e);
	}
}

// Remove all of the <script> and <link> tags that Mozile might have added to the document.
//
function mozileUnload() {
	var f=["mozileConfiguration.js","mozileUnload"];
	mozileDebug(f,2,"Mozile Unloading");

	var removeIDs = new Array();
	removeIDs = mozile_js_files;
	removeIDs.push("mozile-show-tags");
	
	for(var i=0; i<removeIDs.length; i++) {
		var ID = removeIDs[i];
		var remove = mozileTarget.document.getElementById(ID);
		if(remove) {
			remove.parentNode.removeChild(remove);
		}
	}
	
		// chrome has been unloaded
	mozileTarget.mozileChromeLoaded="false";
	
	mozileDebug(f,2,"Chrome Flag: "+ mozileTarget.mozileChromeLoaded);
		// turn off the View Tags button
	var button = document.getElementById("mozile-view-tags");
	button.setAttribute("active","false");
	
	mozileDebug(f,2,"Mozile Unloaded");
}



// Handle before unload events.
// Ignore any that aren't related to the document being edited.
//
function mozileOnBeforeUnload(event) {
	var f=["mozileConfiguration.js","mozileOnBeforeUnload"];
//	mozileDebug(f,2,"Page unload handler.");
	
	try {
	// Make mTarg the document that is being closed
  	var mTarg;
  	if ( event.target.nodeType == Node.DOCUMENT_NODE ) {
  		mTarg = event.target;
  	}
  	else {
  		mTarg = event.target.ownerDocument;
  	}
  
  	if ( "MOZILE_CURRENT_CONFIG" in mTarg ) { //If Mozile injected
  		if(mTarg.MOZILE_CURRENT_CONFIG.saved !== "true") {	//If something to save
  			netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
  			var mozilePrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    		mozileDebug(f,1,"Unload event:"+ event + event.target +" "+ event.target.nodeType +" "+ mTarg.documentURI);
    		if(!mozilePrefs.getBoolPref("mozile.promptOnUnloadBool")) {
    		// if the promptOnUnloadBool preference is "false", do nothing
    			mozileDebug(f,2,"Unloading document.");
    			return false;
    		} else {
				// else pop up an alert asking user if they really want to leave the page.
    			mozileDebug(f,1,"Unsaved changes, unload document?");
    			event.stopPropagation();
    			event.returnValue = false;
    			event.preventDefault();
    			return false;
				}
			}
		}
  }	catch (e) {
		mozileDebug(f,2,"Exception: Nothing to unload? "+e);
	}
	return false;
}

function mozileScourCSS(mozileInjectTarget) {
	var f = ["mozileConfiguration.js","mozileScourCSS"];
	mozileDebug(f,3,"Scouring CSS");
	mozileDebug(f,3,"Number of style sheets: "+mozileInjectTarget.document.styleSheets.length);
	var URL = "";
	for(var	i=0; i<mozileInjectTarget.document.styleSheets.length;i++)	{
		try{
			mozileDebug(f,3,"Style sheet "+ i +" has "+mozileInjectTarget.document.styleSheets[i].cssRules.length+" rules");
			for	(var j=0; j<mozileInjectTarget.document.styleSheets[i].cssRules.length;j++) {
				var	sel	= mozileInjectTarget.document.styleSheets[i].cssRules[j];
				if(sel.selectorText=="mozileconfiguration")	{
					URL = sel.style.backgroundImage;
					URL = URL.substring(4,URL.length-1);
					mozileDebug(f,3,"URL found:"+i+":"+j+":"+URL);
					break;
				}
			}
		} catch(e) {
			// styleSheet is inaccessible, don't worry about it
			mozileDebug(f,2,"Problem with style sheet number "+i+"? "+e);
		}
	}
	return URL;
}

// returns an absolute URL computed from a (possibly relative) URL and a starting point (relativeTo)
function mozileAbsoluteURL(URL, relativeTo) {
	var f = ["mozileConfiguration.js","mozileAbsoluteURL"];
	mozileDebug(f,2,"Start with: "+relativeTo+"  "+URL);
	if(URL.search("://")>0) {
		mozileDebug(f,2,"URL contains '://', so assume it's absolute.");
		return URL;
	}
	relativeTo += " "; // make sure it's a string

  // Remove query string part of already absolute part
  var queryPos = relativeTo.lastIndexOf("?");
  if (queryPos > -1)
    relativeTo = relativeTo.substring(0, queryPos);

  // Check if given URL is relative to host root or current document
  if (URL.charAt(0) == "/") {
    // Make URL relative
    URL = URL.substring(1);

    // Find beginning of hostname
    var hostPos = relativeTo.search("://")+3;
    if (hostPos < 0)
      hostPos = 0;

    // Find end of hostname
    hostPos = relativeTo.indexOf("/", hostPos);
    relativeTo = relativeTo.substring(0, hostPos) + "/";
  }

	var last = relativeTo.lastIndexOf("/");
  var dir = relativeTo.substring(0,last+1);
  	// just tacking the relative URL onto the directory of the reference URL seems to work fine: i.e. http://overton.ca/james/mozile/mozile1/../mozileTest.html becomes http://overton.ca/james/mozile/mozile1/mozileTest.html
	mozileDebug(f,2,"Try: "+ dir + URL);
	URL = dir+URL;
	return URL;
}

function mozileLoadConfig (mozileInjectTarget, headnode) {
	var f = ["mozileConfiguration.js","mozileLoadConfig"];
	mozileDebug(f,2,"Load Config starting");
	
	/* Look	for	an RSD declaration and load	configuration if present */

	var	rsdtype	= "application/rsd+xml";
//	var	rsdns =	"http://archipelago.phrasewise.com/rsd";
	var	presetDisallowXPath	= "//setting[@name='savePresetDisallow']";
	var	methodXPath	= "//setting[@name='saveMethod']";
	var	methodDisallowXPath	= "//setting[@name='saveMethodDisallow']";
	var	contentXPath = "//setting[@name='saveContent']";
	var	contentDisallowXPath = "//setting[@name='saveContentDisallow']";
	var httpSavePathXPath = "//setting[@name='httpSavePath']";
	var httpSavePathModifyXPath = "//setting[@name='httpSavePathModify']";
	var	pasteModeXPath = "//setting[@name='pasteMode']";
	var	mesXPath = "//setting[@name='mes']";
	var	tagCSSXPath	= "//setting[@name='tagCSS']";
	var	rc, cc, rsdhref, rsdloaded, xmlDoc;
	
	// Important Objects, hold all the configuration info for the page
	rc = mozileInjectTarget.document.MOZILE_RSD_CONFIG =	new	Object();
	cc = mozileInjectTarget.document.MOZILE_CURRENT_CONFIG =	new	Object();
	
	// first check in CSS
	var CSS_URL = mozileScourCSS(mozileInjectTarget);
	if(CSS_URL && CSS_URL!="") {	
		mozileDebug(f,2,"Using configuration from CSS file, href="+CSS_URL);
		rsdhref = CSS_URL;
	} 
	// if that fails, check for a <link>
	else {
		mozileDebug(f,2,"CSS config not found. Checking for an RSD <link...>");	
		// Grab	the	head element and look for the RSD link
		// XXX Moz supports	XPath in HTML now, we could	replace	this w/	XPath
		var	links =	headnode.getElementsByTagName("link");
		for	(var i=0; i	< links.length;	i++) {
			// Find	out	if the link	element	has	a type attr	for	RSD
			var	l =	links[i];
			if (l.hasAttribute("type") && l.getAttribute("type") ==	rsdtype) {
				rsdhref	= l.getAttribute("href");
				mozileDebug(f,2,"RSD link found, href="+rsdhref);
			}
		}
	}
	
	rsdloaded = false;
	if ( rsdhref!=undefined && rsdhref!='' ) {
		rsdhref=mozileAbsoluteURL(rsdhref, mozileInjectTarget.document.location); // make sure the URL is absolute
		mozileDebug(f,2,"Loading RSD at "+rsdhref);
		try {
  		xmlDoc = mozileInjectTarget.document.implementation.createDocument("", "", null);
			xmlDoc.async = false;
			rsdloaded = xmlDoc.load(rsdhref);
			if ( !rsdloaded ) {
					mozileDebug(f,1,"RSD file load has failed");
			}
		} catch (e){
			mozileDebug(f,1,"Exception: "+e);
		}
	}
	
	if ( rsdloaded ) {
		mozileDebug(f,2,"RSD file loaded");

		// If the rsd.xml has a namespace decl, the XPath breaks.
		var	evaluator =	new	XPathEvaluator();
		
		var	savePresetDisallow = evaluator.evaluate(presetDisallowXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveContent = evaluator.evaluate(contentXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveContentDisallow = evaluator.evaluate(contentDisallowXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveMethod = evaluator.evaluate(methodXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveMethodDisallow = evaluator.evaluate(methodDisallowXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	httpSavePath = evaluator.evaluate(httpSavePathXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	httpSavePathModify = evaluator.evaluate(httpSavePathModifyXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	pasteMode	= evaluator.evaluate(pasteModeXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	mes	= evaluator.evaluate(mesXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	tagCSS = evaluator.evaluate(tagCSSXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		
		savePresetDisallow = mozileValidateConfigList( "savePreset", savePresetDisallow );
		saveContent = mozileValidateConfig( "saveContent", saveContent );
		saveContentDisallow = mozileValidateConfigList( "saveContent", saveContentDisallow );
		saveMethod = mozileValidateConfig( "saveMethod", saveMethod );
		saveMethodDisallow = mozileValidateConfigList( "saveMethod", saveMethodDisallow );
		httpSavePathModify = mozileValidateConfig( "httpSavePathModify", httpSavePathModify );
		

		httpSavePath = mozileAbsoluteURL(httpSavePath,rsdhref);
		if ( mes!="XHTML" ) { // make sure there are absolute URLs
			mes = mozileAbsoluteURL(mes,rsdhref);
		}
		if ( tagCSS!="XHTML" ) {
			tagCSS = mozileAbsoluteURL(tagCSS,rsdhref);
		}

		// Store the values from the RSD file
		// Store the same RSD values to the MOZILE_CURRENT_CONFIG array
		cc.saved = "true";
		cc.preset = "page";
		cc.rsdhref = rc.rsdhref = rsdhref;
		cc.savePresetDisallow = rc.savePresetDisallow = savePresetDisallow;
		cc.saveContent = rc.saveContent =	saveContent;
		cc.saveContentDisallow = rc.saveContentDisallow = saveContentDisallow;
		cc.saveMethod = rc.saveMethod = saveMethod;
		cc.saveMethodDisallow = rc.saveMethodDisallow = saveMethodDisallow;
		cc.httpSavePath = rc.httpSavePath = httpSavePath;
		cc.httpSavePathModify = rc.httpSavePathModify = httpSavePathModify;
		cc.pasteMode = rc.pasteMode = pasteMode;
		cc.mes = rc.mes = mes;
		cc.tagCSS = rc.tagCSS = tagCSS;
		mozileDebug(f,1,"RSD Configuration completed.");
	}
	else { // default to the built in XHTML Editing Scheme.
		mozileDebug(f,2,"No configuration information found, defaults will be applied");
		// Store the default values in MOZILE_CURRENT_CONFIG.
		cc.saved = "true";
		cc.preset = "user";
		cc.rsdhref = "";
		cc.savePresetDisallow = "";
		cc.saveContent = "page";
		cc.saveContentDisallow = "";
		cc.saveMethod = "display";
		cc.saveMethodDisallow = "";
		cc.httpSavePath = "";
		cc.httpSavePathModify = "true";
		cc.pasteMode = ""; //value?
		cc.mes = "XHTML";
		cc.tagCSS = "XHTML";
	}
	mozileLoadMES(mozileInjectTarget, cc.mes);
}


var mozileValidConfigs = new Object();
mozileValidConfigs.savePreset = new Array("user","all");
mozileValidConfigs.saveContent = new Array("page", "current", "current-ns");
mozileValidConfigs.saveMethod = new Array("display", "local", "post", "webdav" );
mozileValidConfigs.httpSavePathModify = new Array("true", "false");
//mozileSaveOptions.pasteMode = new Array();


//
// Validate a single configuration setting.
//
// All whitespace is removed BEFORE validating.
// Not case sensitive.
// Returns the default value if an invalid value is supplied.
//
function mozileValidateConfig( config, value ) {
	var f = ["mozileConfiguration.js","mozileValidateConfig"];

	value = value.toLowerCase();
	var re = /\s/g;
	value = value.replace( re, "" );
	if ( config in mozileValidConfigs ) {
		var j = mozileValidConfigs[config].length;
		for( var i=0; i < j; i++ ) {
			if (mozileValidConfigs[config][i] == value) {
				return value;
			}	
		}
		//If not found, return default value.
		mozileDebug(f,1,"Invalid RSD setting: name="+config+" value=["+value+"]");
		return mozileValidConfigs[config][0];
	} else {
		return "";
	}
}

// Validate a comma separated list of settings.
//
// All whitespace is removed BEFORE validating.
// Not case sensitive.
// Ignores duplicates.
//
// Returns zero length array if invalid.
//
//
function mozileValidateConfigList( config, csl ) {
	var f = ["mozileConfiguration.js","mozileValidateConfigList"];

	var a = new Array();
	csl = csl.toLowerCase();
	var re = /\s/g;
	csl = csl.replace( re, "" );
	if ( config in mozileValidConfigs ) {
		var c = mozileValidConfigs[config].length;
		a = new Array(c);
		var values = csl.split(",");
		var v = values.length;
		for ( var n=0; n < v; n++ ) {
			for( var i=0; i < c; i++ ) {
				if (mozileValidConfigs[config][i] == values[n]) {
					a[i] = mozileValidConfigs[config][i];
					break;
				}	
			}
			if ( i >= c ) { //If invalid setting
				mozileDebug(f,1,"Invalid RSD setting: name="+config+" value=["+values[n]+"]");
			}
		}
	}
	return a;
} 

